const should = require("should");
const getTestEnv = require("./fixtures/lib.js").getTestEnv;

const stuff = ["cheerio", "babel-generator", "babylon"];

describe("Heavy node modules", (done) => {

    it("Should bundle cheerio", (done) => {
        getTestEnv({
            "index.js": `
            var cheerio = require("cheerio");
            let $ = cheerio.load('<h2 class="title">Hello world</h2>')
            $('h2.title').text('Hello there!')
            $('h2').addClass('welcome')
            exports.str = $.html();
            `,
        }, "**/*.js").then(root => {

            let result = root.FuseBox.import("./index");

            result.should.deepEqual({ str: '<h2 class="title welcome">Hello there!</h2>' });
            done();
        }).catch(done);
    });


    it("Should bundle babel-generator", (done) => {
        getTestEnv({
            "hello.js": `
            var generator = require('babel-generator');
            exports.data = generator;
            `,
        }, "**/*.js").then(root => {
            let result = root.FuseBox.import("./hello");
            result.data.CodeGenerator.should.ok;
            //require("core-js/library/fn/symbol")
            //result.should.deepEqual({ str: '<h2 class="title welcome">Hello there!</h2>' });
            done();
        }).catch(done);
    })

    it("Should partially require problematic module from core-js", (done) => {
        getTestEnv({
            "hello.js": `
            exports.data = require("core-js/library/fn/symbol");
            `,
        }, "**/*.js").then(root => {
            let result = root.FuseBox.import("./hello");
            result.data.keyFor.should.ok;
            done();
        }).catch(done);
    });

    it("Should bundle core-js", (done) => {
        getTestEnv({
            "hello.js": `
            exports.data = require("core-js");
            `,
        }, "**/*.js").then(root => {
            let result = root.FuseBox.import("./hello");
            result.data.version.should.be.ok;
            // res/ult.data.keyFor.should.ok;
            done();
        }).catch(done);
    });

    it("Should bundle babylon", (done) => {

        getTestEnv({
            "hello.js": `
        var generator = require('babel-generator')
        var babylon = require('babylon')
        const code = 'class Example {}';
        const ast = babylon.parse(code);

        exports.test = generator.default(ast, {}, code);
            `,
        }, "**/*.js").then(root => {

            let result = root.FuseBox.import("./hello");
            result.test.code.should.equal("class Example {}");

            done();
        }).catch(done);
    })
})